@@ -1,7 +1,7 @@
 /*
  *  Benchmark demonstration program
  *
- *  Copyright (C) 2006-2010, Brainspark B.V.
+ *  Copyright (C) 2006-2011, Brainspark B.V.
  *
  *  This file is part of PolarSSL (http://www.polarssl.org)
  *  Lead Maintainer: Paul Bakker <polarssl_maintainer at polarssl.org>
@@ -41,23 +41,53 @@
 #include "polarssl/arc4.h"
 #include "polarssl/des.h"
 #include "polarssl/aes.h"
+#include "polarssl/blowfish.h"
 #include "polarssl/camellia.h"
+#include "polarssl/gcm.h"
 #include "polarssl/rsa.h"
 #include "polarssl/timing.h"
+#include "polarssl/havege.h"
+#include "polarssl/ctr_drbg.h"
 
-#define BUFSIZE 1024
+#define BUFSIZE         1024
+#define HEADER_FORMAT   "  %-15s :  "
 
-static int myrand( void *rng_state )
+static int myrand( void *rng_state, unsigned char *output, size_t len )
 {
+    size_t use_len;
+    int rnd;
+
     if( rng_state != NULL )
         rng_state  = NULL;
 
-    return( rand() );
+    while( len > 0 )
+    {
+        use_len = len;
+        if( use_len > sizeof(int) )
+            use_len = sizeof(int);
+
+        rnd = rand();
+        memcpy( output, &rnd, use_len );
+        output += use_len;
+        len -= use_len;
+    }
+
+    return( 0 );
 }
 
 unsigned char buf[BUFSIZE];
 
-int main( void )
+#if !defined(POLARSSL_TIMING_C)
+int main( int argc, char *argv[] )
+{
+    ((void) argc);
+    ((void) argv);
+
+    printf("POLARSSL_TIMING_C not defined.\n");
+    return( 0 );
+}
+#else
+int main( int argc, char *argv[] )
 {
     int keysize;
     unsigned long i, j, tsc;
@@ -71,20 +101,35 @@ int main( void )
 #endif
 #if defined(POLARSSL_AES_C)
     aes_context aes;
+#if defined(POLARSSL_GCM_C)
+    gcm_context gcm;
+#endif
+#endif
+#if defined(POLARSSL_BLOWFISH_C)
+    blowfish_context blowfish;
 #endif
 #if defined(POLARSSL_CAMELLIA_C)
     camellia_context camellia;
 #endif
-#if defined(POLARSSL_RSA_C)
+#if defined(POLARSSL_RSA_C) && defined(POLARSSL_BIGNUM_C) &&    \
+    defined(POLARSSL_GENPRIME)
     rsa_context rsa;
 #endif
+#if defined(POLARSSL_HAVEGE_C)
+    havege_state hs;
+#endif
+#if defined(POLARSSL_CTR_DRBG_C)
+    ctr_drbg_context    ctr_drbg;
+#endif
+    ((void) argc);
+    ((void) argv);
 
     memset( buf, 0xAA, sizeof( buf ) );
 
     printf( "\n" );
 
 #if defined(POLARSSL_MD4_C)
-    printf( "  MD4       :  " );
+    printf( HEADER_FORMAT, "MD4" );
     fflush( stdout );
 
     set_alarm( 1 );
@@ -100,7 +145,7 @@ int main( void )
 #endif
 
 #if defined(POLARSSL_MD5_C)
-    printf( "  MD5       :  " );
+    printf( HEADER_FORMAT, "MD5" );
     fflush( stdout );
 
     set_alarm( 1 );
@@ -116,7 +161,7 @@ int main( void )
 #endif
 
 #if defined(POLARSSL_SHA1_C)
-    printf( "  SHA-1     :  " );
+    printf( HEADER_FORMAT, "SHA-1" );
     fflush( stdout );
 
     set_alarm( 1 );
@@ -132,7 +177,7 @@ int main( void )
 #endif
 
 #if defined(POLARSSL_SHA2_C)
-    printf( "  SHA-256   :  " );
+    printf( HEADER_FORMAT, "SHA-256" );
     fflush( stdout );
 
     set_alarm( 1 );
@@ -148,7 +193,7 @@ int main( void )
 #endif
 
 #if defined(POLARSSL_SHA4_C)
-    printf( "  SHA-512   :  " );
+    printf( HEADER_FORMAT, "SHA-512" );
     fflush( stdout );
 
     set_alarm( 1 );
@@ -164,7 +209,7 @@ int main( void )
 #endif
 
 #if defined(POLARSSL_ARC4_C)
-    printf( "  ARC4      :  " );
+    printf( HEADER_FORMAT, "ARC4" );
     fflush( stdout );
 
     arc4_setup( &arc4, tmp, 32 );
@@ -182,7 +227,7 @@ int main( void )
 #endif
 
 #if defined(POLARSSL_DES_C)
-    printf( "  3DES      :  " );
+    printf( HEADER_FORMAT, "3DES" );
     fflush( stdout );
 
     des3_set3key_enc( &des3, tmp );
@@ -198,7 +243,7 @@ int main( void )
     printf( "%9lu Kb/s,  %9lu cycles/byte\n", i * BUFSIZE / 1024,
                     ( hardclock() - tsc ) / ( j * BUFSIZE ) );
 
-    printf( "  DES       :  " );
+    printf( HEADER_FORMAT, "DES" );
     fflush( stdout );
 
     des_setkey_enc( &des, tmp );
@@ -218,7 +263,7 @@ int main( void )
 #if defined(POLARSSL_AES_C)
     for( keysize = 128; keysize <= 256; keysize += 64 )
     {
-        printf( "  AES-%d   :  ", keysize );
+        printf( "  AES-CBC-%d     :  ", keysize );
         fflush( stdout );
 
         memset( buf, 0, sizeof( buf ) );
@@ -237,12 +282,35 @@ int main( void )
         printf( "%9lu Kb/s,  %9lu cycles/byte\n", i * BUFSIZE / 1024,
                         ( hardclock() - tsc ) / ( j * BUFSIZE ) );
     }
+#if defined(POLARSSL_GCM_C)
+    for( keysize = 128; keysize <= 256; keysize += 64 )
+    {
+        printf( "  AES-GCM-%d     :  ", keysize );
+        fflush( stdout );
+
+        memset( buf, 0, sizeof( buf ) );
+        memset( tmp, 0, sizeof( tmp ) );
+        gcm_init( &gcm, tmp, keysize );
+
+        set_alarm( 1 );
+
+        for( i = 1; ! alarmed; i++ )
+            gcm_crypt_and_tag( &gcm, GCM_ENCRYPT, BUFSIZE, tmp, 12, NULL, 0, buf, buf, 16, tmp );
+
+        tsc = hardclock();
+        for( j = 0; j < 4096; j++ )
+            gcm_crypt_and_tag( &gcm, GCM_ENCRYPT, BUFSIZE, tmp, 12, NULL, 0, buf, buf, 16, tmp );
+
+        printf( "%9lu Kb/s,  %9lu cycles/byte\n", i * BUFSIZE / 1024,
+                        ( hardclock() - tsc ) / ( j * BUFSIZE ) );
+    }
+#endif
 #endif
 
 #if defined(POLARSSL_CAMELLIA_C)
     for( keysize = 128; keysize <= 256; keysize += 64 )
     {
-        printf( "  CAMELLIA-%d   :  ", keysize );
+        printf( "  CAMELLIA-CBC-%d:  ", keysize );
         fflush( stdout );
 
         memset( buf, 0, sizeof( buf ) );
@@ -263,11 +331,96 @@ int main( void )
     }
 #endif
 
-#if defined(POLARSSL_RSA_C)
+#if defined(POLARSSL_BLOWFISH_C)
+    for( keysize = 128; keysize <= 256; keysize += 64 )
+    {
+        printf( "  BLOWFISH-CBC-%d:  ", keysize );
+        fflush( stdout );
+
+        memset( buf, 0, sizeof( buf ) );
+        memset( tmp, 0, sizeof( tmp ) );
+        blowfish_setkey( &blowfish, tmp, keysize );
+
+        set_alarm( 1 );
+
+        for( i = 1; ! alarmed; i++ )
+            blowfish_crypt_cbc( &blowfish, BLOWFISH_ENCRYPT, BUFSIZE, tmp, buf, buf );
+
+        tsc = hardclock();
+        for( j = 0; j < 4096; j++ )
+            blowfish_crypt_cbc( &blowfish, BLOWFISH_ENCRYPT, BUFSIZE, tmp, buf, buf );
+
+        printf( "%9lu Kb/s,  %9lu cycles/byte\n", i * BUFSIZE / 1024,
+                        ( hardclock() - tsc ) / ( j * BUFSIZE ) );
+    }
+#endif
+
+#if defined(POLARSSL_HAVEGE_C)
+    printf( HEADER_FORMAT, "HAVEGE" );
+    fflush( stdout );
+
+    havege_init( &hs );
+
+    set_alarm( 1 );
+    for( i = 1; ! alarmed; i++ )
+        havege_random( &hs, buf, BUFSIZE );
+
+    tsc = hardclock();
+    for( j = 1; j < 1024; j++ )
+        havege_random( &hs, buf, BUFSIZE );
+
+    printf( "%9lu Kb/s,  %9lu cycles/byte\n", i * BUFSIZE / 1024,
+                    ( hardclock() - tsc ) / ( j * BUFSIZE ) );
+#endif
+
+#if defined(POLARSSL_CTR_DRBG_C)
+    printf( HEADER_FORMAT, "CTR_DRBG (NOPR)" );
+    fflush( stdout );
+
+    if( ctr_drbg_init( &ctr_drbg, myrand, NULL, NULL, 0 ) != 0 )
+        exit(1);
+
+    set_alarm( 1 );
+    for( i = 1; ! alarmed; i++ )
+        if( ctr_drbg_random( &ctr_drbg, buf, BUFSIZE ) != 0 )
+            exit(1);
+
+    tsc = hardclock();
+    for( j = 1; j < 1024; j++ )
+        if( ctr_drbg_random( &ctr_drbg, buf, BUFSIZE ) != 0 )
+            exit(1);
+
+    printf( "%9lu Kb/s,  %9lu cycles/byte\n", i * BUFSIZE / 1024,
+                    ( hardclock() - tsc ) / ( j * BUFSIZE ) );
+
+    printf( HEADER_FORMAT, "CTR_DRBG (PR)" );
+    fflush( stdout );
+
+    if( ctr_drbg_init( &ctr_drbg, myrand, NULL, NULL, 0 ) != 0 )
+        exit(1);
+
+    ctr_drbg_set_prediction_resistance( &ctr_drbg, CTR_DRBG_PR_ON );
+
+    set_alarm( 1 );
+    for( i = 1; ! alarmed; i++ )
+        if( ctr_drbg_random( &ctr_drbg, buf, BUFSIZE ) != 0 )
+            exit(1);
+
+    tsc = hardclock();
+    for( j = 1; j < 1024; j++ )
+        if( ctr_drbg_random( &ctr_drbg, buf, BUFSIZE ) != 0 )
+            exit(1);
+
+    printf( "%9lu Kb/s,  %9lu cycles/byte\n", i * BUFSIZE / 1024,
+                    ( hardclock() - tsc ) / ( j * BUFSIZE ) );
+#endif
+
+#if defined(POLARSSL_RSA_C) && defined(POLARSSL_BIGNUM_C) &&    \
+    defined(POLARSSL_GENPRIME)
     rsa_init( &rsa, RSA_PKCS_V15, 0 );
     rsa_gen_key( &rsa, myrand, NULL, 1024, 65537 );
 
-    printf( "  RSA-1024  :  " );
+    printf( HEADER_FORMAT, "RSA-1024" );
     fflush( stdout );
     set_alarm( 3 );
 
@@ -279,7 +432,7 @@ int main( void )
 
     printf( "%9lu  public/s\n", i / 3 );
 
-    printf( "  RSA-1024  :  " );
+    printf( HEADER_FORMAT, "RSA-1024" );
     fflush( stdout );
     set_alarm( 3 );
 
@@ -296,7 +449,7 @@ int main( void )
     rsa_init( &rsa, RSA_PKCS_V15, 0 );
     rsa_gen_key( &rsa, myrand, NULL, 2048, 65537 );
 
-    printf( "  RSA-2048  :  " );
+    printf( HEADER_FORMAT, "RSA-2048" );
     fflush( stdout );
     set_alarm( 3 );
 
@@ -308,7 +461,7 @@ int main( void )
 
     printf( "%9lu  public/s\n", i / 3 );
 
-    printf( "  RSA-2048  :  " );
+    printf( HEADER_FORMAT, "RSA-2048" );
     fflush( stdout );
     set_alarm( 3 );
 
@@ -325,7 +478,7 @@ int main( void )
     rsa_init( &rsa, RSA_PKCS_V15, 0 );
     rsa_gen_key( &rsa, myrand, NULL, 4096, 65537 );
 
-    printf( "  RSA-4096  :  " );
+    printf( HEADER_FORMAT, "RSA-4096" );
     fflush( stdout );
     set_alarm( 3 );
 
@@ -337,7 +490,7 @@ int main( void )
 
     printf( "%9lu  public/s\n", i / 3 );
 
-    printf( "  RSA-4096  :  " );
+    printf( HEADER_FORMAT, "RSA-4096" );
     fflush( stdout );
     set_alarm( 3 );
 
@@ -354,10 +507,11 @@ int main( void )
 
     printf( "\n" );
 
-#ifdef WIN32
+#if defined(_WIN32)
     printf( "  Press Enter to exit this program.\n" );
     fflush( stdout ); getchar();
 #endif
 
     return( 0 );
 }
+#endif /* POLARSSL_TIMING_C */
